package com.example.fdsampleeth;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;

import com.datecs.fiscalprinter.FiscalPrinterException;
import com.datecs.fiscalprinter.tza.FMP10TZA;

import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Toast;
import android.app.Activity;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.DialogInterface.OnKeyListener;

public class MainActivity extends Activity {
	private static final int REQUEST_ENABLE_BT = 1;
	private static final int REQUEST_DEVICE = 2;
	
	private static final UUID SPP_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
	
	private BluetoothAdapter mBtAdapter;
	private BluetoothSocket mBtSocket;
	private FMP10TZA mFMP;
	
	private final Handler mHandler = new Handler();
	
	private interface MethodInvoker {
		public void invoke() throws IOException;
	}
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
                
        mBtAdapter = BluetoothAdapter.getDefaultAdapter();
        if (mBtAdapter != null) {
        	if (mBtAdapter.isEnabled()) {
        		selectDevice();
        	} else {
        		enableBluetooth();
        	}
        } else {
        	Toast.makeText(this, R.string.msg_bluetooth_is_not_supported, Toast.LENGTH_SHORT).show();
        	finish();
        	return;
        }        
        
        findViewById(R.id.btn_panic_operation).setOnClickListener(new OnClickListener() {			
			@Override
			public void onClick(View v) {
				panicOperation();
			}
		});
        
        findViewById(R.id.btn_non_fiscal_receipt).setOnClickListener(new OnClickListener() {			
			@Override
			public void onClick(View v) {
				nonFiscalReceipt();
			}
		});
        
        findViewById(R.id.btn_fiscal_receipt).setOnClickListener(new OnClickListener() {			
			@Override
			public void onClick(View v) {
				fiscalReceipt();
			}
		});
        
        findViewById(R.id.btn_z_report).setOnClickListener(new OnClickListener() {            
            @Override
            public void onClick(View v) {
                printZReport();
            }
        });
    }
    
    @Override
    public void onDestroy() {
    	super.onDestroy();    	    	
    	disconnect();    	
    }
    
    @Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
			case REQUEST_ENABLE_BT: {
				if (resultCode == RESULT_OK) {
					selectDevice();
				} else {
					finish();
				}
				break;
			}
			case REQUEST_DEVICE: {
				if (resultCode == RESULT_OK) {
					String address = data.getStringExtra(DeviceActivity.EXTRA_ADDRESS);
					connect(address);
				} else {
					finish();
				}
				break;
			}
		}
	}

	private void enableBluetooth() {
    	Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
	    startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);	    
    }
    
    private void selectDevice() {
    	Intent selectDevice = new Intent(this, DeviceActivity.class);
	    startActivityForResult(selectDevice, REQUEST_DEVICE);	    
    }
    
    private void postToast(final String text) {    	
    	mHandler.post(new Runnable() {			
			@Override
			public void run() {
				Toast.makeText(getApplicationContext(), text, Toast.LENGTH_SHORT).show();
			}
		});
    }
    
    public void connect(final String address) {
    	invokeHelper(new MethodInvoker() {			
			@Override
			public void invoke() throws IOException {
				final BluetoothDevice device = mBtAdapter.getRemoteDevice(address);
				final BluetoothSocket socket = device.createRfcommSocketToServiceRecord(SPP_UUID);
				socket.connect();
				
				mBtSocket = socket;
				final InputStream in = socket.getInputStream();
				final OutputStream out = socket.getOutputStream();					
				mFMP = new FMP10TZA(in, out);
				postToast("Connected");
				
			}
		});    	
	}        
     
    public synchronized void disconnect() {
    	if (mBtSocket != null) {
    		try {
				mBtSocket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
    	}
    }
    
    private void invokeHelper(final MethodInvoker invoker) {
    	final ProgressDialog dialog = new ProgressDialog(this);
    	dialog.setMessage(getString(R.string.msg_please_wait));
		dialog.setOnKeyListener(new OnKeyListener() {					
			@Override
			public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
				return true;
			}
		});
		dialog.show();
		
    	final Thread t = new Thread(new Runnable() {			
			@Override
			public void run() {				
				try {
					invoker.invoke();				
		    	} catch (final IOException e) { // Critical exception
		    		e.printStackTrace();		    		
		    		disconnect();   		
		    		selectDevice();
		    	} finally {
		    		dialog.dismiss();
		    	}
			}
		});    	
    	t.start();
    }
    
    private void panicOperation() {
    	invokeHelper(new MethodInvoker() {			
			@Override
			public void invoke() throws IOException {
                try {
                    mFMP.customCommand(39, "");
                } catch (FiscalPrinterException fex) {
                }
                try {
                    mFMP.customCommand(60, "");
                } catch (FiscalPrinterException fex) {
                }
                try {
                    mFMP.customCommand(53, "\t\t");
                } catch (FiscalPrinterException fex) {
                }                
                try {
                    mFMP.customCommand(56, "");
                } catch (FiscalPrinterException fex) {
                }
			}
		});
    }
    
    private void nonFiscalReceipt() {
    	invokeHelper(new MethodInvoker() {			
			@Override
			public void invoke() throws IOException {
			    mFMP.customCommand(38,"");
                mFMP.customCommand(42,"* SIMPLE NON FISCAL TEXT* \t");
                mFMP.customCommand(39,"");		
			}
		});
    }
    
    private void fiscalReceipt() {
    	invokeHelper(new MethodInvoker() {		
			@Override
			public void invoke() throws IOException {
			    mFMP.customCommand(48,"1\t0000\t1\t0\t");
                mFMP.customCommand(59,"123123123123\t");
                mFMP.customCommand(49,"Bananas\t1\t5.000\t3.000\t\t\t");
                mFMP.customCommand(49,"PLU#1111\\t1\\t1.234\\t2.000\\t\\t\\t");
                mFMP.customCommand(53,"\t\t");
                mFMP.customCommand(56,"");
			}
		});
    }
    
    private void printZReport() {
        invokeHelper(new MethodInvoker() {
            @Override
            public void invoke() throws IOException {
                mFMP.customCommand(69, "Z\t");
            }
        });
    }
}
